#ifndef MPP_CANVAS_H
#define MPP_CANVAS_H

#include<cstring>

#include <FL/Fl_Overlay_Window.H>
#include <FL/Fl_Group.H>
#include <FL/Fl_Menu_Button.H>
#include <FL/fl_draw.H>

#include <mpp/mpp.h>
#include <mpp/scale.h>

class Coord_out
{
public:
    virtual void update_coord(double, double ) {};
    virtual ~Coord_out() {};
};

namespace mpp
{
class Shape;
class Canvas_: public Fl_Overlay_Window, public Widget
{
public:
    Fl_Menu_Button mb;
    Canvas_(int x, int y, int w, int h);
    void update();
    int handle(int event );
    void scale(Scale * scale__, const char * x, const char * y) {
        scale_=scale__;
        strcpy(xlabel,x);
        strcpy(ylabel,y);
    }
    void scale(Scale & scale__, const char * x, const char * y) {
        scale(& scale__, x, y);
    }
    void add_shape(Shape *);
    void add_shape(Shape &);
    Fl_Widget *coord_out;
    // what to do on click
    void (*on_click_)(double x, double y);
private:
    void draw_overlay();
    Scale *scale_;
    std::list<Shape*> shape_list;
    void draw();
    // bufers for x, y labels
    char xlabel[256], ylabel[256];
    char char_buf[256];	 // bufer for toolabar label
    //~Canvas_();
};
class Canvas: public Window, public Fl_Window
{
public:
    Canvas_ canvas_;
    Fl_Group toolbar;
    // as a window
    Canvas(int w=500, int h=500, const char * name="Canvas");
    Canvas(const char * name);
    // as a widget
    Canvas(int x, int y, int w=500, int h=500, const char * name="Canvas");
    void show() {
        Fl_Window::show();
    }
    void hide() {
        Fl_Window::hide();
    }
    const char * label() {
        return Fl_Window::label();
    };
    Fl_Color color() const {
        return canvas_.color();
    }
    void color(Fl_Color color) {
        canvas_.color(color);
    }
    //void resizable(Fl_Widget *box){Fl_Window::resizable(box);}
    //void resizable(Fl_Widget &box){Fl_Window::resizable(box);}

    /*
      int handle(int event){
      std::cout << "Canvas::handle\n";
      return Fl_Window::handle(event);
      };
    */
    void add_shape(Shape *);
    void add_shape(Shape &);
    void set_on_click(void (*on_click)(double x, double y)) {
        canvas_.on_click_=on_click;
    }
    //void update();
    Scale *scale();
    //void on_move(void (*f)(void));
    void scale(Scale * scale__, const char *x="x = ", const char *y="y = ") {
        canvas_.scale(scale__, x, y);
    }
    void scale(Scale & scale__, const char *x="x = ", const char *y="y = ") {
        scale(&scale__,x,y);
    }

    //double part_left, part_right, part_up, part_down;  // visible part of canvas, zoom
private:
    //std::list<Shape*> shape_list;
    //Scale *scale_;
    //void draw();
    void init();
    //void draw_overlay();
    //char xlabel[256], ylabel[256];  // bufers for x, y labels
    //char char_buf[256];				// bufer for toolabar label
};
};
#endif

