#ifndef MPP_MPP_H_
#define MPP_MPP_H_

#include <list>

#include <FL/fl_ask.H>

#ifdef POSIX__
#include <pthread.h>
#include <unistd.h>
#endif

#ifdef NTTHREADS__
#include <windows.h>
#include <winbase.h>
#endif

#include <FL/Fl.H>

namespace mpp
{
typedef double Float;
class Group;
class Widget
{
public:
    virtual void update() {
        need_update=false;
    };
    virtual void ask_update() {
        need_update=true;
    };
    virtual void set_value() {};
    Widget();
    virtual ~Widget() {};
protected:
    Group * mpp_parent_;
    bool need_update;
};

class Group: public Widget
{
public:
    static Group * current_;
    Group();
    virtual ~Group() {};
    void add(Widget *);
    void mpp_begin();
    void mpp_end();
    void update();
    void ask_update();
    void set_value();
private:
    std::list<Widget *> widget_list;
};

class Window: public Group
{
public:
    Window();
    virtual ~Window() {};
    virtual void show() {};
    virtual void hide() {};
    virtual const char *label() {
        return 0;
    };
private:
};

// for update, ...
std::list<Widget*>& group_list();
// for mpp manager
std::list<Window*>& window_list();

void update();
void ask_update();
void set_value();
void start();
void stop();
inline bool may_i_repeat();
inline void ask_suspend();
inline void ask_continue();
int run();
inline void lock();
inline void unlock();
inline void msleep(double t);
#ifdef POSIX__
extern pthread_t tid;
#endif
#ifdef NTTHREADS__
extern HANDLE thread;
extern DWORD tid;
#endif
extern bool ask_stop;
extern volatile bool suspend_flag;
extern bool calc;
// new thread
extern void (*Calc_func)(void);
// same thread
extern void (*Calc_func_)(void);
// same thread
extern void (*Calc_func_3)(void);
void set_work(void (*f)(void));
void set_work2(void (*f)(void));
// same thread via timer
void set_work3(void (*f)(void), double sleep_time);
extern double sleep_time;
// task priorities
extern int priority_hour;
extern int priority_day;
extern int priority_week;
inline void ask_set_priority(int);
extern int priority_value;	 // private
extern bool priority_changed;// private
// private
inline void set_priority(int);

extern double update_time;
inline void set_update_time(double t) {
    update_time=t;
};

extern bool visible_v;
inline bool visible() {
    return visible_v;
}
inline bool set_visible(bool v) {
    return visible_v=v;
}

//extern const char * mppg_info;
//const char * mppg_info="mppg";
//inline void mppg_about(Fl_Widget * , void *){fl_message(mppg_info);}
inline void mppg_about(Fl_Widget *, void *) {
    fl_message("mppg");
}


// implementation
#ifdef LINUX
#include <sys/resource.h>
void set_priority(int prio) {
    setpriority(PRIO_PROCESS,PRIO_PROCESS,prio);
}
#else
void set_priority(int) {}
#endif

void ask_set_priority(int prior) {
    mpp::priority_value=prior;
    mpp::priority_changed=true;
}

#include <FL/Fl.H>
void lock() {
    Fl::lock();
}
void unlock() {
    Fl::unlock();
}
bool may_i_repeat() {
    while(suspend_flag) ;
    if (priority_changed) {
        set_priority(priority_value);
        priority_changed=false;
    }
    return !ask_stop;
};
void ask_suspend() {
    suspend_flag=true;
}
void ask_continue() {
    suspend_flag=false;
}

void msleep(double t) {
#ifdef NTTHREADS__
    Sleep(int(t*1e3));
#else
    #include <unistd.h>
    usleep(int(t*1e6));
#endif
}

};
#endif
