#ifndef MPP_SCALE_H
#define MPP_SCALE_H

#include <iostream>
#include <cmath>

#include <mpp/mpp.h>

inline double POW(int p) {		 //raise value to the p power
    double tmp=1;
    if (p>0) {
        for (int i=0; i<p; i++)
            tmp*=10;
        return tmp;
    }
    if (p<0) {
        for (int i=0; i>p; i--)
            tmp*=0.1;
        return tmp;
    }
    return tmp;
}


namespace mpp
{
#define KEEP_RATIO 1
#define NO_KEEP_RATIO 0
class Scale
{
public:
    bool aspect_ratio;
    Float x1, x2, y1, y2;
    Scale(Float x1, Float x2, Float y1, Float y2, bool ar=false): aspect_ratio(ar), x1(x1), x2(x2), y1(y1), y2(y2) {};
    inline void set(Float x1, Float x2, Float y1, Float y2, bool ar=false);
    inline void xleft(Float x) {
        x1=x;
    }
    inline Float xleft(void) {
        return x1;
    }
    inline void xright(Float x) {
        x2=x;
    }
    inline Float xright(void) {
        return x2;
    }
    inline void ydown(Float y) {
        y1=y;
    }
    inline Float ydown(void) {
        return y1;
    }
    inline void yup(Float y) {
        y2=y;
    }
    inline Float yup(void) {
        return y2;
    }
    operator const Scale* () const {
        return this;
    }
};

inline std::ostream & operator<<(std::ostream & os, const Scale &scale) {
    return (os << scale.x1 << ' ' << scale.x2 << ' ' << scale.y1 << ' ' << scale.y2);
}

inline std::istream & operator>>(std::istream & is, Scale &scale) {
    return (is >> scale.x1 >> scale.x2 >> scale.y1 >> scale.y2);
}

struct scale_t
{
    Float X;
    Float x;
    Float Y;
    Float y;
};

extern scale_t scale_;

inline int transx(Float x);
inline int transy(Float y);
inline int scalex(Float x);
inline int scaley(Float y);
inline Float c2fx(int x);
inline Float c2fy(int y);
void        set_scale(const Scale *);
inline void set_scale(const Scale &s) {
    set_scale(&s);
};
inline std::pair<double,double> auto_scale(double val);

/////////////////////////////////////////////////////////////////////////////
// Implementation
/////////////////////////////////////////////////////////////////////////////
void Scale::set(Float x1_, Float x2_, Float y1_, Float y2_, bool ar) {
    x1=x1_;
    x2=x2_;
    y1=y1_;
    y2=y2_;
    aspect_ratio=ar;
}
int transx(Float x) {
    return (int)(scale_.X*x+scale_.x);
};
int transy(Float y) {
    return (int)(scale_.Y*y+scale_.y);
};
int scalex(Float x) {
    return (int)(scale_.X*x);
};
int scaley(Float y) {
    return (int)(scale_.Y*y);
};
Float c2fx(int x) {
    return (x-scale_.x)/scale_.X;
}
Float c2fy(int y) {
    return (y-scale_.y)/scale_.Y;
}
std::pair<double,double> auto_scale(double val) {
    val*=1.07;
    const int sz=8;
    static double ar[sz][2]= { {
            12,2
        }
        ,
        {
            15,2.5
        }
        ,
        {
            20,4
        }
        ,
        {
            30,5
        }
        ,
        {
            40,5
        }
        ,
        {
            50,10
        }
        ,
        {
            70,10
        }
        ,
        {
            100,20
        }
    };
    int p=(int)floor(log(val)/log(10.))-1;
    double tmp=val/POW(p);
    std::pair<double,double> kk(1,1);
    for (int i=0; i<sz; i++)
        if (tmp<=ar[i][0]) {
            double tmp2=POW(p);
            kk.first=ar[i][0]*tmp2;
            kk.second=ar[i][1]*tmp2;
            return kk;
        }
    return kk;
}

};
#endif
