#include <cmath>
#include <sstream>
#include <iostream>
#include <deque>
#include <vector>

#include <string.h>

#ifdef TIFF_SUPPORT
#include <tiffio.h>
#include <mpp/grabbing.h>
#endif

#include <FL/Fl_Window.H>
#include <FL/Fl.H>

#ifdef PRINT
#include <FL/Fl_Printer.H>
//#include <FL/Fl_PS_Printer.H>
//#include <FL/fl_printer_chooser.H>
#endif

#include <FL/Fl_File_Chooser.H>
#include <FL/fl_draw.H>

#include <mpp/manager.h>
#include <mpp/canvas.h>
#include <mpp/draw.h>

#include <mpp/scale.h>
//#include <mpp/grabbing.h>
#include <mpp/file_dialog.h>

mpp::scale_t mpp::scale_;		 // namespace member
Fl_Window * mpp::draw_window=0;

#ifdef PRINT
static void print_callback(Fl_Widget *, void *w) {
    Fl_Widget * g = (Fl_Widget *)w;
    Fl_Printer * p = new Fl_Printer();
    if (!p->start_job(1)) {
        p->origin(36, -72);
        p->start_page();
        p->print_window(g->window());
        p->end_page();
        p->end_job();
    }
    delete p;
}

/*
static void printps_callback(Fl_Widget *, void *w) {
	const int wpoints=504;
	Fl_Overlay_Window * g = (Fl_Overlay_Window *)w;

	char * filename = mpp::file_dialog("Save to file","EPS Files (*.eps)\0*.eps\0All Files (*.*)\0*.*\0","1.eps");
	if(!filename ) return;
	if (!strlen(filename) ) return;
	FILE * f = fopen(filename,"w");
	if(!f) return;
	Fl_Printer * p = new Fl_PS_Printer(f, 3, (590-wpoints)/2, (792-wpoints*g->h()/g->w())/2, wpoints, wpoints*g->h()/g->w());
	p->place(g, 0, 0, wpoints, wpoints*g->h()/g->w(), FL_ALIGN_CENTER);
	Fl_Device * c = p->set_current();
	Fl_Color color=g->color();
	g->color(FL_WHITE);
	fl_draw(g);
	g->color(color);
	c->set_current();
	delete p;
	fclose(f);					 //we need close file here
};
*/
#endif

#ifdef TIFF_SUPPORT
static void printtiff_callback(Fl_Widget *, void *w) {

    char * filename = mpp::file_dialog("Save to file","Tiff Files\0(*.tif)\0All Files\0(*.*)\0","1.tif");
    if(!filename) return;
    FILE * f = fopen(filename,"w");
    if(!f) return;
    mpp::grabbing_tif((Fl_Window*)w, filename, COMPRESSION_NONE);

    fclose(f);					 //we need close file here
};

static void printtiff_deflate_callback(Fl_Widget *, void *w) {

    char * filename = mpp::file_dialog("Save to file","Tiff Files\0(*.tif)\0All Files\0(*.*)\0","1d.tif");
    if(!filename) return;
    FILE * f = fopen(filename,"w");
    if(!f) return;
    mpp::grabbing_tif((Fl_Window*)w, filename, COMPRESSION_DEFLATE);

    fclose(f);					 //we need close file here
};
#endif

static void manager_callback(Fl_Widget *, void *) {
    mpp::manager->show();
}


mpp::Canvas_::Canvas_(int x, int y, int w, int h):Fl_Overlay_Window(x,y,w,h), mb(0,0,w,h) {
    color(FL_BLACK);
#ifdef  PRINT
    mb.add("&Print ...", FL_ALT+'p', print_callback, (Fl_Overlay_Window *)this, 0);
    // mb.add("Print &eps ...", FL_ALT+'e', printps_callback, (Fl_Overlay_Window *)this, 0);
#endif
#ifdef TIFF_SUPPORT
    mb.add("Print &tiff ...", FL_ALT+'t', printtiff_callback, this, 0);
    mb.add("Print tiff &deflate ...", FL_ALT+'d', printtiff_deflate_callback, this, 0);
#endif
    mb.add("&Manager", FL_ALT+'w', manager_callback, 0, 0);
    mb.type(Fl_Menu_Button::POPUP3);
    scale_=0;
    on_click_=0;
    coord_out=0;
    cursor(FL_CURSOR_CROSS);
    resizable(&mb);
    end();
}


void mpp::Canvas_::draw_overlay() {
    fl_color(FL_RED);
    int x_=Fl::event_x(), y_=Fl::event_y();
    if (window()->window()) {
        y_-=30;
    };
    fl_line(x_,0,x_,h());
    fl_line(0,y_,w(),y_);
}


int mpp::Canvas_::handle(int event ) {
    int ret=Fl_Overlay_Window::handle(event);

    switch(event) {
    case FL_MOVE:
    {
        if (scale_) {
            //std::stringstream ss;
            //ss << xlabel << c2fx(Fl::event_x()) << ",\t" << ylabel << c2fy(Fl::event_y());
            //toolbar.label(strcpy(char_buf,(ss.str()).c_str()));
            //redraw();
            redraw_overlay();
            if (coord_out) {
                std::stringstream ss;
                // !!! bad place
                draw_window=(Fl_Window*)this;
                set_scale(scale_);
                ss << xlabel << c2fx(Fl::event_x()) << ",\t" << ylabel << c2fy(Fl::event_y());
                coord_out->label(strcpy(char_buf,(ss.str()).c_str()));
            }
        }
        break ;
    }
    case FL_PUSH:
    {
        if (scale_ and on_click_) {
            set_scale(scale_);
            on_click_(c2fx(Fl::event_x()), c2fy(Fl::event_y()));
        }
        break;
    }
        /*
        case FL_ENTER:
        	if (window()){ window()->cursor(FL_CURSOR_CROSS);}
        		else { fl_cursor(FL_CURSOR_CROSS);}
        	//fl_cursor(FL_CURSOR_CROSS);
        	return 0;
        	//return Fl_Group::handle(event);

        case FL_LEAVE:
        	if (window() ) window()->cursor(FL_CURSOR_DEFAULT);
        		else fl_cursor(FL_CURSOR_DEFAULT);
        	//fl_cursor(FL_CURSOR_DEFAULT);
        	return 0;
        	//return Fl_Group::handle(event);
        */
    }
    return ret;
}


void mpp::Canvas_::add_shape(mpp::Shape *sh) {
    shape_list.push_back(sh);
}


void mpp::Canvas_::add_shape(mpp::Shape &sh) {
    shape_list.push_back(&sh);
}


void mpp::Canvas_::draw() {
    Fl_Overlay_Window::draw();
    draw_window=(Fl_Window*)this;// !!! bad place
    fl_push_clip(0, 0, w(), h());
    for (std::list<mpp::Shape *>::const_iterator p=shape_list.begin(); p!=shape_list.end(); ++p) {
        (*p)->draw();
    }
    fl_pop_clip();
}


void mpp::Canvas_::update() {
    if (need_update) {
        Widget::update();
        redraw();
    }
}


void mpp::Canvas::init() {
    toolbar.align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
    toolbar.box(FL_DOWN_BOX);
    canvas_.coord_out=&toolbar;
    //toolbar.label("jkdfjglkdfjgdlfk");
    end();
    mpp_end();
    Fl_Window::resizable(canvas_);
}


mpp::Canvas::Canvas(int w, int h, const char * name):Fl_Window(w, h, name), canvas_(0,0,w,h-30), toolbar(0,h-30,w,30) {
    //mpp::Canvas::Canvas(int w, int h, const char * name):Fl_Window(w, h, name), toolbar(0,h-30,w,30){
    Canvas::init();
    if (mpp::visible()) show();
    //	fl_cursor(FL_CURSOR_CROSS);
}


mpp::Canvas::Canvas(const char *name):Fl_Window(500, 500, name), canvas_(0,0,500,500-30), toolbar(0,500-30,500,30) {
    //mpp::Canvas::Canvas(const char *name):Fl_Window(500, 500, name), toolbar(0,500-30,500,30){
    Canvas::init();
    if (mpp::visible()) show();
    //	fl_cursor(FL_CURSOR_CROSS);
}


mpp::Canvas::Canvas(int x, int y, int w, int h, const char * name):Fl_Window(x, y, w, h, name), canvas_(0,0,w,h-30), toolbar(0,h-30,w,30) {
    //mpp::Canvas::Canvas(int x, int y, int w, int h, const char * name):Fl_Window(x, y, w, h, name), toolbar(x,y+h-30,w,30){
    Canvas::init();
    //show();
}


//mpp::Canvas::scale

/*
void mpp::Canvas::draw(){
	Fl_Window::draw();
	draw_window=(Fl_Window*)this;   // !!! bad place
	fl_push_clip(0, 0, w(), h()-30);
	for (std::list<mpp::Shape *>::const_iterator p=shape_list.begin(); p!=shape_list.end(); ++p) {
		(*p)->draw();
	}
	fl_pop_clip();
}
*/

void mpp::Canvas::add_shape(mpp::Shape *sh) {
    canvas_.add_shape(sh);
}


void mpp::Canvas::add_shape(mpp::Shape &sh) {
    add_shape(&sh);
}


/*
int mpp::Canvas::handle(int event ){
	switch(event) {

		case FL_MOVE:{
			if (scale_){
				std::stringstream ss;
				ss << xlabel << c2fx(Fl::event_x()) << ",\t" << ylabel << c2fy(Fl::event_y());
				toolbar.label(strcpy(char_buf,(ss.str()).c_str()));
				redraw();
				redraw_overlay();
				//return Fl_Window::handle(event);
			}
			return 1;
		}

		default:
			return Fl_Window::handle(event);
	}
}
*/

/*
void mpp::Canvas::update(){
	if (need_update){
		Window::update();
		redraw();
		//std::stringstream ss;
		//ss << "x = " << c2fx(Fl::event_x()-x()) << ",\t y = " << c2fy(Fl::event_y()-y());
	  /////  toolbar.label(strcpy(char_buf,(ss.str()).c_str()));
	}
}
*/
void mpp::set_scale(const Scale * sc) {
    if (sc->aspect_ratio) {
        if ( ((double)draw_window->h()-30)/draw_window->w() > fabs((sc->y2 - sc->y1)/(sc->x2 - sc->x1)) ) {
            scale_.X=draw_window->w()/(sc->x2-sc->x1);
            scale_.Y=-fabs(scale_.X*(sc->y2 - sc->y1))/(sc->y2 - sc->y1);
        }
        else {
            scale_.Y=-(draw_window->h())/(sc->y2-sc->y1);
            scale_.X=fabs(scale_.Y*(sc->x2 - sc->x1))/(sc->x2 - sc->x1);
        }
        scale_.x=draw_window->w()*0.5-scale_.X*0.5*(sc->x1+sc->x2);
        scale_.y=(draw_window->h())*0.5-scale_.Y*0.5*(sc->y1+sc->y2);
    }
    else {
        scale_.X=draw_window->w()/(sc->x2-sc->x1);
        //scale_.X=draw_window->w()/( (sc->x2-sc->x1)*
        //        (1.0-draw_window->part_left-draw_window->part_right) );
        scale_.x=-draw_window->w()*sc->x1/(sc->x2-sc->x1);
        scale_.Y=-(draw_window->h())/(sc->y2-sc->y1);
        scale_.y=(draw_window->h())+draw_window->h()*sc->y1/(sc->y2-sc->y1);
    };
}
