#include <FL/Fl.H>
#include <FL/fl_draw.H>

#include <mpp/draw.h>
#include <mpp/canvas.h>

int mpp::point_size=0;
int mpp::point_size_b=1;

mpp::Circle::Circle(mpp::Float x_, mpp::Float y_, mpp::Float R_, Fl_Color color_, const mpp::Scale * sc) {
    x=x_;
    y=y_;
    R=R_;
    color=color_;
    scale=sc;
}


void mpp::Circle::draw()const
{
    set_scale(scale);
    fl_color(color);
    fl_circle(transx(x), transy(y), scalex(R));
}


void mpp::plot(double(*f)(double), double xl, double xr, int n) {
    double xx=(xr-xl)/(n-1);
    for (int i=0; i<n; ++i) {
        point(xl+xx*i, f(xl+xx*i));
    }
}


//Hist
//***************************************************

mpp::Hist::Hist(unsigned int nc_, double xmin_, double xmax_,
                Fl_Color color1, const mpp::Scale *scale, int legs_): chan(nc_), scale(scale) {
    //nc=nc_;
    xmin=xmin_;
    xmax=xmax_;
    color=color1;
    legs=legs_;
    for (unsigned int i=0, nc=chan.size(); i<nc; i++) chan[i]=0;
};

/*
mpp::Hist::Hist(unsigned int nc_, double xmin_, double xmax_,
  Fl_Color color1, const mpp::Scale &scale, int legs_): chan(nc_), scale(&scale){
	//nc=nc_;
	xmin=xmin_;
	xmax=xmax_;
	color=color1;
	legs=legs_;
	for (unsigned int i=0, nc=chan.size(); i<nc;i++) chan[i]=0;
};
*/

void mpp::Hist::add(const double *x, unsigned int sz) {
    double dx=(xmax-xmin)/chan.size();
    unsigned int k;
    for (unsigned int i=0; i<sz; i++)
        if ((x[i]>=xmin)&&(x[i]<=xmax)) {
            k=(unsigned int)((x[i]-xmin)/dx);
            chan[k]+=1;
        }
};

void mpp::Hist::set(const double *x, unsigned int sz) {
    for (unsigned int k=0, nc=chan.size(); k<nc; k++) chan[k]=0;
    add(x,sz);
}


void mpp::Hist::draw()const
{
    set_scale(scale);
    fl_color(color);

    double xc=xmin, dx=(xmax-xmin)/chan.size();
    if (legs)
        for (unsigned int k=0, nc=chan.size(); k<nc; k++) {
            fl_line(transx(xc),transy(0),transx(xc),transy(chan[k]));
            fl_line(transx(xc),transy(chan[k]),transx(xc+dx),transy(chan[k]));
            xc+=dx;
            fl_line(transx(xc),transy(0),transx(xc),transy(chan[k]));
        }
    else {
        for (unsigned int k=0, nc=chan.size(); k<nc; k++) {
            if (k>0) fl_line(transx(xc),transy(chan[k-1]),transx(xc),transy(chan[k]));
            else fl_line(transx(xc),transy(0),transx(xc),transy(chan[k]));
            fl_line(transx(xc),transy(chan[k]),transx(xc+dx),transy(chan[k]));
            xc+=dx;
            if (k<(nc-1)) fl_line(transx(xc),transy(chan[k]),transx(xc),transy(chan[k+1]));
            else fl_line(transx(xc),transy(0),transx(xc),transy(chan[k]));
        }
    }
}


//Roll
//***************************************************

mpp::Roll::Roll(Fl_Color color1, const mpp::Scale *s) {
    color=color1;
    scale=s;
};

/*
mpp::Roll::Roll(Fl_Color color1, const mpp::Scale &s){
	color=color1;
	scale=&s;
};
*/

void mpp::Roll::add(double x, double y) {
    //mpp::lock();
    roll_q.push_back(std::pair<mpp::Float, mpp::Float>(x,y));
    double t=roll_q.back().first-(scale->x2-scale->x1);
    while ( roll_q.front().first<t ) roll_q.pop_front();
    //mpp::unlock();
};

void mpp::Roll::draw()const
{
    set_scale(scale);
    fl_color(color);
    double tmp=0;
    if (roll_q.size()>0) tmp=roll_q.back().first-(scale->x2-scale->x1);
    for (std::deque<std::pair<mpp::Float, mpp::Float> >::const_iterator p=roll_q.begin(); p!=roll_q.end(); ++p)
        point(p->first-tmp,p->second);
}


void mpp::Roll::reset() {
    roll_q.resize(0);
}


// Orbit
//***************************************************

mpp::Orbit::Orbit(int sz_all_, int sz_head_, Fl_Color color_h, Fl_Color color_t, const Scale* sc) {
    if (sz_all_>sz_head_) {
        sz_all=sz_all_;
        sz_head=sz_head_;
    }
    else {
        sz_all=sz_head_;
        sz_head=sz_head_;
    }
    color_head=color_h;
    color_tail=color_t;
    scale=sc;
}


/*
mpp::Orbit::Orbit(int sz_all_, int sz_head_, Fl_Color color_h, Fl_Color color_t, const Scale& sc){
	if (sz_all_>sz_head_) {
		sz_all=sz_all_;
		sz_head=sz_head_;
	}
	else{
		sz_all=sz_head_;
		sz_head=sz_head_;
	}
	color_head=color_h;
	color_tail=color_t;
	scale=&sc;
}
*/

void mpp::Orbit::draw()const
{
    if (pvec.size()>0) {
        set_scale(scale);
        fl_color(color_head);
        for (unsigned int i=0; i<sz_head; i++) pointb(pvec[i].first, pvec[i].second);
        fl_color(color_tail);
        {
            for (unsigned int i=sz_head; i<sz_all; i++) point(pvec[i].first, pvec[i].second);
        }
    }
}


void mpp::Orbit::add(double x, double y) {
    if (!(pvec.size()>0)) {		 // first using
        pvec.resize(sz_all);
        for (unsigned int i=0; i<sz_all; i++) pvec[i]=std::pair<Float, Float>(x, y);
        p1=0;
        p2=sz_head;
    }
    else {
        p1++;
        p2++;
        if (!(p1<sz_head)) p1=0;
        if (!(p2<sz_all))  p2=sz_head;
        pvec[p2]=pvec[p1];
        pvec[p1]=std::pair<Float, Float>(x, y);
    }
}


void mpp::Orbit::reset() {
    pvec.resize(0);
}


// Bitmap
mpp::Bitmap::Bitmap(const double *rmap1, int w1, int h1, double c11, double c21, const Scale *sc1) {
    sc=sc1;
    rmap=rmap1;
    w=w1;
    h=h1;
    c1=c11;
    c2=c21;
    map = 0;
};

/*
mpp::Bitmap::Bitmap(const double *rmap1, int w1, int h1, double c11, double c21, const Scale &sc1){
   sc=&sc1;
   rmap=rmap1;
   w=w1; h=h1;
   c1=c11; c2=c21;
   map = 0;
};
*/

mpp::Bitmap::~Bitmap() {
    delete[] map;
}


void mpp::Bitmap::draw()const
{
    set_scale(sc);

    if ( ( (int)(scale_.X*w) !=W ) || ( (int)(scale_.Y*h) != H) ) {
        H=abs((int)(scale_.Y*h));
        W=abs((int)(scale_.X*w));
        delete[] map;
        map = new uchar[H*W];
    }
    double tmp=255.0/(c2-c1);
    for (int i=0; i<H; i++)
        for(int j=0; j<W; j++)
            map[i*W+j]=(uchar)((c2-rmap[w*(int)(i*(1.0*h)/H)+(int)(j*(1.0*w)/W)])*tmp);

    //Fl::visual(FL_RGB);
    fl_draw_image_mono(map, transx(0), transy(0), W, H);
}


void mpp::Plot_a::draw()const
{
    fl_color(color);
    set_scale(scale);
    for (int i=0; i<sz; i++)
        mpp::point(x[i], y[i]);
}


void mpp::Plot_v::draw()const
{
    fl_color(color);
    set_scale(scale);
    int sz=x.size();
    for (int i=0; i<sz; i++)
        point(x[i], y[i]);
}


void mpp::Plot_l::draw()const
{
    fl_color(color);
    set_scale(scale);
    for (std::list<std::pair<mpp::Float, mpp::Float> >::const_iterator p=pointl.begin(); p!=pointl.end(); p++)
        point(p->first, p->second);
}
