#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/fl_draw.H>

#ifdef PRINT
#include <FL/Fl_Printer.H>
#include <FL/Fl_PostScript.H>
#endif

#include<tiffio.h>
#include<stdlib.h>

#include <mpp/grabbing.h>
#include <mpp/canvas.h>
#include <mpp/mpp.h>

/*
	unsigned char * grabbing(Fl_Window *, int x, int y, int dx, int dy);
	unsigned char * grabbing(Fl_Window *);

	void grabbing(Fl_Window *, int x, int y, int dx, int dy, const char * fname, int compr=COMPRESSION_DEFLATE);
	void grabbing(Fl_Window *, const char * fname, int compr=COMPRESSION_DEFLATE);
*/

#ifdef PRINT

void mpp::grabbing_png(Fl_Widget *w, const char * fname, int rezolution) {
    mpp::grabbing_eps(w, "1_q_a_z.ps");
    char s_tmp[256];
    char s2_tmp[256];
    sprintf(s2_tmp,"%d",rezolution);
    strcpy(s_tmp, "ps2eps 1_q_a_z.ps 1>/dev/null 2>/dev/null ; ");
#ifdef WIN32
    strcat(s_tmp,"gswin32c.exe -dBATCH -dNOPAUSE -dEPSCrop -sDEVICE=png16m -r");
#else
    strcat(s_tmp,"gs -dBATCH -dNOPAUSE -dEPSCrop -sDEVICE=png16m -r");
#endif
    strcat(s_tmp,s2_tmp);
    strcat(s_tmp," -sOutputFile=");
    strcat(s_tmp,fname);
    strcat(s_tmp," 1_q_a_z.eps 1>/dev/null 2>/dev/null  ; rm -f 1_q_a_z.ps 1_q_a_z.eps");
    system(s_tmp);
}


void mpp::grabbing_eps(Fl_Widget *w, const char * filename) {
    FILE * f = fopen(filename,"w");
    if(!f) return;
    mpp::lock();
    Fl_PostScript_File_Device * p= new Fl_PostScript_File_Device;
    p->start_job(f, 1);

    int l, t, r, b;
    p->margins(&l,&t,&r,&b);
    p->origin(-l, -t -50);

    //int w_, h_;
    //p->printable_rect(&w_, &h_);
    //p->scale(21/2.54*72/w_);



    p->start_page();
    p->print_widget(w);
    p->end_page();
    p->end_job();
    delete p;
    mpp::unlock();
    fclose(f);					 //we need close file here
}
#endif

void grabbing_(Fl_Window * win, const char * fname, int comp) {
    static unsigned int csz=0;
    static unsigned char * bm=0;

    unsigned int dx=win->w(), dy=win->h();
    if (csz<3*dx*dy) {
        csz=3*dx*dy;
        if (bm) delete[] bm;
        bm = new unsigned char[csz];
    }
    mpp::lock();
    win->make_current();
    fl_read_image(bm, 0, 0, dx, dy, 0);

    TIFF* img;

    if((img=TIFFOpen(fname,"w"))==0) {
        TIFFError(0,"TIFFOpen");
        exit(1);
    }
    //TIFFSetField(img,TIFFTAG_FILLORDER,FILLORDER_LSB2MSB);
    //TIFFSetField(img,TIFFTAG_FILLORDER,FILLORDER_MSB2LSB);

    //TIFFSetField(img,TIFFTAG_PIXARLOGDATAFMT,PIXARLOGDATAFMT_8BITABGR);

    TIFFSetField(img,TIFFTAG_BITSPERSAMPLE,8);
    TIFFSetField(img,TIFFTAG_SAMPLESPERPIXEL,3);
    TIFFSetField(img,TIFFTAG_PLANARCONFIG,1);
    //TIFFSetField(img,TIFFTAG_PLANARCONFIG,PLANARCONFIG_SEPARATE);
    TIFFSetField(img,TIFFTAG_PHOTOMETRIC,PHOTOMETRIC_RGB);
    TIFFSetField(img,TIFFTAG_IMAGEWIDTH,dx);
    TIFFSetField(img,TIFFTAG_IMAGELENGTH,dy);
    TIFFSetField(img,TIFFTAG_COMPRESSION,comp);

    //TIFFSetField(img,TIFFTAG_ROWSPERSTRIP,dx);
    //TIFFSetField(img,TIFFTAG_FILLORDER,FILLORDER_MSB2LSB);

    //TIFFWriteEncodedStrip(img, 0, bm, dx*dy*3);

    for(size_t i=0; i<dy; i++)
        TIFFWriteScanline(img, bm+i*dx*3, i);
    mpp::unlock();
    TIFFClose(img);
}
