#include <iostream>
#include <vector>
#include <list>

#include <mpp/mpp_window.h>
#include <mpp/draw.h>

#include <FL/Fl.H>

#include "rand_m.h"

using namespace std;
using namespace mpp;

size_t colony_sz=2000;
size_t animal_sz=200;
vector<double> rez(animal_sz);
vector<double> del(colony_sz);
size_t step=0;
double alpha=1;
double delta=0.01;
double s2=0.01;

class Animal{
public:
	vector<double> v;
	double fitness()const;	
	Animal(size_t sz=0):v(sz), valid(false){};	
	//~Animal(){};
	void mut(double delta, size_t pos);
	mutable bool valid;  // ???
private:	
	mutable double quality;
};

ostream & operator<<(ostream & os, const Animal & a){
	for (size_t i=0, sz=a.v.size(); i<sz; i++)
		cout << a.v[i] << ' ';
	return os;
}

void crossover(const Animal & m, const Animal & f, Animal & c){
	size_t pos=Irand(m.v.size()+1);
	
	for (size_t i=0, sz=m.v.size(); i<sz; i++){
		//c.v[1]=m.v[1];
		//cout << i << endl;
		if (i<pos) c.v[i]=m.v[i];
		else c.v[i]=f.v[i];
	}	

	c.valid=false;
}

bool operator<(const Animal& A1, const Animal& A2){
	if (A1.fitness()<A2.fitness())	return true;
	return false;
}


class Colony{
public:
	list<Animal> la; 
	Colony(size_t c, size_t a);
private:
};

void ff(Colony& colony){           	
	list<Animal>::iterator f=colony.la.begin(),c=colony.la.end();	
	list<Animal>::iterator m=f++;
	c--;
	while (f!=c && m!=c){
		crossover(*m, *f, *c);
		m++; f++; c--;
	}
	
	
	for (int j=0; j<200; j++){ 		
		list<Animal>::iterator p=colony.la.begin();
		size_t animal_num=Irand(colony.la.size()), i=0;
		while (i!=animal_num){
			p++; i++;
		}		   		
		size_t pos=Irand(colony.la.front().v.size());
		p->mut(delta, pos);                          		
	}		
	colony.la.sort();             	
}                

Colony colony(colony_sz, animal_sz);

Scale scale(0, animal_sz, -2, 2);
Scale scale2(0, animal_sz, -0.02, 0.02);
Scale scale3(0, colony_sz, 0.0, 0.004);
Grid grid(0, 50, -2, 0.25, scale);
Mpp_window win;

void f(){
	while (!may_i_repeat()){
		step++;		
		ff(colony);		
		size_t i=0;
		for (list<Animal>::const_iterator p=colony.la.begin(); p!=colony.la.end(); p++){			
			del[i]=0;
			for (size_t j=0, sz=p->v.size(); j<sz; j++)			
				del[i]+=fabs(p->v[j]-colony.la.front().v[j]);
			del[i]/=p->v.size();
			i++;
		}		
		if (step%20 == 0)	ask_update();
	}
}

void ud(){
	set_scale(scale);
	set_color(FL_WHITE);
	for (size_t i=0; i<animal_sz; i++)
		point(i,rez[i]);		
	set_color(FL_YELLOW);
	for (size_t i=0; i<animal_sz; i++)
		pointb(i,colony.la.front().v[i]);
	set_scale(scale2);
	set_color(FL_RED);
	for (size_t i=0; i<animal_sz; i++)
		pointb(i,colony.la.back().v[i]-colony.la.front().v[i]);				
	set_scale(scale3);
	set_color(FL_GREEN);
	for (size_t i=0; i<colony_sz; i++)
		pointb(i,del[i]);				
}



int main(){
	for (size_t i=0; i<animal_sz; i++)
		rez[i]=sin(4*2*M_PI*i/animal_sz)+Drand()*0.1;
		//rez[i]=2.*i*i/(animal_sz*animal_sz)+Drand()*0.1;
	
//	win.table.add_item("d_phi", d_phi);
	win.table.add_item("alpha", alpha);
	win.table.add_item("delta", delta);
	//win.table.add_item("step", step);
	win.canvas.add_shape(grid);
	win.canvas.add_shape(new User_draw(ud));	
	set_work(f);
	start();
	run();
	/*
	for (list<Animal>::iterator p=colony.la.begin(); p!=colony.la.end(); p++)
		cout << *p << endl << endl;
	cout << "Test\n";
   */
	return 0;
}


// Implementation
double Animal::fitness()const{
	if (!valid){
		double s=0;
		for (int i=0, sz=v.size(); i<sz; i++)
			s+=(v[i]-rez[i])*(v[i]-rez[i]);
		for (int i=1, sz=v.size()-1; i<sz; i++){
			double tmp=2*v[i]-v[i-1]-v[i+1];
			s+=tmp*tmp*alpha;
		}
		quality=s;
		valid=true;
	}
	return quality;	
}

void Animal::mut(double delta, size_t pos){	
						v[pos  ]+=Drand()*delta;
	if (pos+1<v.size()) v[pos+1]+=Drand()*delta;
	if (pos>=1) 		v[pos-1]+=Drand()*delta;
	if (pos+2<v.size()) v[pos+2]+=Drand()*delta;
	if (pos>=2) 		v[pos-2]+=Drand()*delta;
	if (pos+3<v.size()) v[pos+3]+=Drand()*delta;
	if (pos>=3) 		v[pos-3]+=Drand()*delta;
	valid=false;	
}

Colony::Colony(size_t l, size_t a):la(l){
	for (list<Animal>::iterator p=la.begin(); p!=la.end(); p++)
		p->v.resize(a);
	la.sort();
}
